@@ -3,25 +3,27 @@
 Author: David J. Morfe
 Application Name: GeoLiberator
 Functionality Purpose: Instill data quality upon address data
-Version: Alpha 0.2.3
+Version: Beta 0.2.9
 '''
-#7/15/19
+#8/12/19
 
 import re
 import sys
 import time
 
-##t0 = time.process_time_ns()
-
-#Account for word house numbers
-#Account for '331/River/NJ/Rd' and post cardinal direction
+#Account for post cardinal direction
 #Account for '&' and 'STS' and multiple street types
 #Option to append borough, state, zip, based on argument
 #Create custom address formatter
 
+reason = ["Invalid parse argument given"]
+
 class AddressError(BaseException):
     pass
 
+class ArgumentError(Exception):
+    pass
+
 class Unbuffered(object):
    def __init__(self, stream):
        self.stream = stream
@@ -33,7 +35,6 @@ class Unbuffered(object):
        self.stream.flush()
    def __getattr__(self, attr):
        return getattr(self.stream, attr)
-
 sys.stdout = Unbuffered(sys.stdout)
 
 class GeoLiberator:
@@ -62,13 +63,13 @@ class GeoLiberator:
                        "Pennsylvania": "PA", "Rhode Island": "RI", "South Carolina": "SC", "South Dakota": "SD",
                        "Tennessee": "TN", "Texas": "TX", "Utah": "UT", "Vermont": "VT", "Virginia": "VA",
                        "Washington": "WA", "West Virginia": "WV", "Wisconsin": "WI", "Wyoming": "WY"}
-        self.streetTypes = {"ROAD": ["ROAD","RD","RO"],
+        self.streetTypes = {"HEIGHTS": ["HEIGHTS","HTS"],
+                            "ROAD": ["ROAD","RD","RO"],
                             "AVENUE": ["AVENUE","AVEN","AVE","AV","AE"],
                             "DRIVE": ["DRIVE","DRIV","DR"],
                             "PLACE": ["PLACE","PLAC","PLCE","PLC","PL"],
                             "BOULEVARD": ["BOULEVARD","BLVD","BOUL","BLV","BO"],
                             "COURT": ["COURT","CRT","CT"],
-                            "HEIGHTS": ["HEIGHTS","HTS"],
                             "PARKWAY": ["PARKWAY","PKWAY","PKWY","PWAY","PWY","PKY"],
                             "HIGHWAY": ["HIGHWAY","HWAY","HWY"],
                             "EXPRESSWAY": ["EXPRESSWAY","EXPRESWY","EXPRESWAY","EXPREWAY","EXPWA","EXPWY","EXPY","EXWY","EWY","EXP"],
@@ -108,7 +109,7 @@ class GeoLiberator:
             hold += re.sub(r"[\[\]' ]", '', str(self.streetTypes[typ])) + ','
         self.streetTypesAll = list(hold.strip(',').split(','))
 
-    def getCompass(self, direc):
+    def _get_compass(self, direc):
         if 'N' == direc or "NO" == direc:
             return "NORTH"
         elif 'S' == direc or "SO" == direc:
@@ -120,8 +121,10 @@ class GeoLiberator:
         else:
             return False
 
-    def ordinalAdd(self, num):
+    def _ordinal_add(self, num):
         nn = ''
+        if num == '11' or num == '12' or num == '13':
+            return num + 'th'
         if re.search(r"(?<!1)1$", num):
             nn = num + 'st'
         elif num[-1] == '2':
@@ -132,7 +135,25 @@ class GeoLiberator:
             nn = num + 'th'
         return nn
 
-    def searchCycle(self, g, sF):
+    def _get_state(self, log=''):
+        get = (self.addr).upper(); full_state = '' #Uppercase and create new address to return
+        get = (re.sub(r"[\t!#$@%^*+=`~/]+| +", ' ', get)).strip(' ') #Strip any anomalies
+        for key, val in self.states.items():
+            if re.search(fr"\b{val}\b", get):
+                full_state = key
+        if full_state == '':
+            full_state = "OTHER"
+
+        if log != '': #Write to new or specfied file
+            fileName = re.sub(r"\..+", '', log)
+            if fileName.isdigit() or re.search(r'[\/:*?"<>|]', fileName):
+                fileName = "newly_parsed_states"
+            nf = open(f"{fileName}.txt", 'a')
+            nf.write(full_state + '\n')
+            nf.close()
+        return str(full_state).upper()
+
+    def _search_cycle(self, g, sF):
         new_find = ''
         if sF == False:
             for stre in self.wordTypes: #Check for word/name street types
@@ -158,7 +179,7 @@ class GeoLiberator:
             getStreetPattern3 = re.search(r"(?!\d)?(AVENUE|AVEN\.?|AVE\.?|AV\.?|AE\.?) ([A-Z]|OF ([A-Z]+ )?[A-Z]+)(?=\W|$)", g)
             if getStreetPattern1:
                 if getStreetPattern1.group(4) in self.streetTypes[key] or getStreetPattern1.group(7) in self.streetTypes[key]:
-                    new_find = self.getCompass(getStreetPattern1.group(2)) + ' ' + getStreetPattern1.group(3).strip('. ') + f" {key}"
+                    new_find = self._get_compass(getStreetPattern1.group(2)) + ' ' + getStreetPattern1.group(3).strip('. ') + f" {key}"
                     break
             elif getStreetPattern2:
                 if getStreetPattern2.group(3) in self.streetTypes[key] or getStreetPattern2.group(5) in self.streetTypes[key]:
@@ -168,49 +189,62 @@ class GeoLiberator:
                 new_find = "AVENUE " + getStreetPattern3.group(2)
                 break
         if sF == True and new_find != '':
-            new_find = "SAINT " + str(new_find)
+            cardir = re.search(r"(NORTH|SOUTH|EAST|WEST) ", str(new_find))
+            if cardir:
+                new_find = re.sub(fr"{cardir[1]} ", f"{cardir[1]} SAINT ", new_find)
+            else:
+                new_find = "SAINT " + str(new_find)
         if new_find == '':
             new_find = "OTHER"
         return new_find
 
     def getAddressNum(self, log=''):
         get = (self.addr).upper(); new_addr_num = '' #Uppercase and create new address to return
-        get = (re.sub(r"[\t!#$@%^*+=`~/]| +", ' ', get)).strip(' ') #Strip any anomalies
-        get = re.sub(r"(?<=\d)(ND|RD|TH|RTH)", '', get) #Strip any char of ordinal numbers
-        for val in self.wordTypes:
+        if not get[0].isdigit():
+            for key, val in self.states.items():
+                if re.search(fr"^{val}\b", get):
+                    get = re.sub(fr"^{val}(\W|$)", ' ', get)
+                    get = (re.sub(r"[\t!#$@%^*+=`~/]+| +", ' ', get)).strip(' ') #Strip any anomalies
+                    get = re.sub(r"(?<=2)(ND)|(?<=[4-9]|0)(TH|RTH)", '', get) #Strip any char of ordinal numbers
+                    get = re.sub(r"(?<=[^1]3)(RD)", '', get); get = re.sub(r"(?<=11)(TH)", '', get)
+        else:
+            get = (re.sub(r"[\t!#$@%^*+=`~/]+| +", ' ', get)).strip(' ') #Strip any anomalies
+            get = re.sub(r"(?<=2)(ND)|(?<=[4-9]|0)(TH|RTH)", '', get) #Strip any char of ordinal numbers
+            get = re.sub(r"(?<=[^1]3)(RD)", '', get); get = re.sub(r"(?<=11)(TH)", '', get)
+        for val in self.wordTypes: #Word Street Names
             if type(val) == dict:
                 streA = '|'.join(val["ESPLANADE"])
                 streB = '|'.join(val["BROADWAY"])
                 grab = re.search(fr"(?!\d)?\W?({streA}|{streB})(\W|$)", get); wType = ''
                 if grab:
                     wType = grab.group(1)
-                group1 = fr"(^\d+([- ]\d+)?)(?=[NSEW]\.? ?\d+ ?({wType})\.?(\W|$))"
-                group2 = fr"(^\d+([- ]\d+)?)(?= ((\w+\.? ?)+)({wType})\.?(\W|$))"
-                group3 = r"(?=\d+([ -]\d+)? (AVENUE|AVEN\.?|AVE\.?|AV\.?|AE\.?) ([A-Z]|OF ([A-Z]+ )?[A-Z]+)(?=\W|$))^\d+([- ]\d+)?"
-                gANpat1 = re.search(fr"{group1}|{group2}", get)
-                gANpat2 = re.search(fr"{group3}", get)
-                if gANpat1:
-                    new_addr_num = gANpat1.group().replace(' ', '-')
-                elif gANpat2:
-                    new_addr_num = gANpat2.group().replace(' ', '-')
+                    group1 = fr"(^\d+([- ]\d+)?)(?= ?[NSEW][. ] ?({wType})\.?(\W|$))"
+                    group2 = fr"(^\d+([- ]\d+)?)(?= (NORTH |SOUTH |EAST |WEST )?({val})\.?(\W|$))"
+                    group3 = r"(?=\d+([- ]\d+)? (AVENUE|AVEN\.?|AVE\.?|AV\.?|AE\.?) ([A-Z]|OF ([A-Z]+ )?[A-Z]+)(?=\W|$))^\d+([- ]\d+)?"
+                    gANpat1 = re.search(fr"{group1}|{group2}", get)
+                    gANpat2 = re.search(fr"{group3}", get)
+                    if gANpat1:
+                        new_addr_num = gANpat1.group().replace(' ', '-')
+                    elif gANpat2:
+                        new_addr_num = gANpat2.group().replace(' ', '-')
             else:
-                sType = '|'.join(val)
-                group1 = fr"(^\d+([- ]\d+)?)(?=[NSEW]\.? ?\d+ ?({sType})\.?(\W|$))"
-                group2 = fr"(^\d+([- ]\d+)?)(?= ((\w+\.? ?)+)({sType})\.?(\W|$))"
-                group3 = r"(?=\d+([ -]\d+)? (AVENUE|AVEN\.?|AVE\.?|AV\.?|AE\.?) ([A-Z]|OF ([A-Z]+ )?[A-Z]+)(?=\W|$))^\d+([- ]\d+)?"
+                group1 = fr"(^\d+([- ]\d+)?)(?= ?[NSEW][. ] ?({val})\.?(\W|$))"
+                group2 = fr"(^\d+([- ]\d+)?)(?= (NORTH |SOUTH |EAST |WEST )?({val})\.?(\W|$))"
+                group3 = r"(?=\d+([- ]\d+)? (AVENUE|AVEN\.?|AVE\.?|AV\.?|AE\.?) ([A-Z]|OF ([A-Z]+ )?[A-Z]+)(?=\W|$))^\d+([- ]\d+)?"
                 gANpat1 = re.search(fr"{group1}|{group2}", get)
                 gANpat2 = re.search(fr"{group3}", get)
                 if gANpat1:
                     new_addr_num = gANpat1.group().replace(' ', '-')
                 elif gANpat2:
                     new_addr_num = gANpat2.group().replace(' ', '-')
-        for key, val in self.streetTypes.items():
+        for key, val in self.streetTypes.items(): #Regular Street Names
             sType = '|'.join(val)
-            group1 = fr"(^\d+([- ]\d+)?)(?=[NSEW]\.? ?\d+ ?({sType})\.?(\W|$))"
-            group2 = fr"(^\d+([- ]\d+)?)(?= ((\w+\.? ?)+)({sType})\.?(\W|$))"
-            group3 = r"(?=\d+([ -]\d+)? (AVENUE|AVEN\.?|AVE\.?|AV\.?|AE\.?) ([A-Z]|OF ([A-Z]+ )?[A-Z]+)(?=\W|$))^\d+([- ]\d+)?"
-            gANpat1 = re.search(fr"{group1}|{group2}", get)
-            gANpat2 = re.search(fr"{group3}", get)
+            group = fr"(^\d+([- ]\d+)?)(?= ?[NSEW][. ]([A-Z]+ )+({sType})\.?(\W|$))"
+            group1 = fr"(^\d+([- ]\d+)?)(?= ?[NSEW][. ]? ?\d+ ?({sType})\.?(\W|$))"
+            group2 = fr"(^\d+([- ]\d+)?)(?=( ?(NORTH|SOUTH|EAST|WEST)? )((\w+\.? ?)+)({sType})\.?(\W|$))"
+            group3 = r"(?=\d+([- ]\d+)? (AVENUE|AVEN\.?|AVE\.?|AV\.?|AE\.?) ([A-Z]|OF ([A-Z]+ )?[A-Z]+)(?=\W|$))^\d+([- ]\d+)?"
+            gANpat1 = re.search(fr"{group}|{group1}|{group2}", get)
+            gANpat2 = re.search(group3, get)
             if gANpat1:
                 new_addr_num = gANpat1.group().replace(' ', '-')
             elif gANpat2:
@@ -229,21 +263,30 @@ class GeoLiberator:
 
     def getStreet(self, log=''):
         get = (self.addr).upper(); new_street = ''; saintFlag = False #Uppercase and create new address to return
-        get = (re.sub(r"[\t!#$@%^*+=`~/]| +", ' ', get)).strip(' ') #Strip any anomalies
-        get = re.sub(r"(?<=\d)(ND|RD|TH|RTH)", '', get) #Strip any char of ordinal numbers
+        if '/' in get:
+            for key, val in self.states.items():
+                if re.search(fr"\b{val}\b", get):
+                    get = re.sub(fr"(^|\W){val}(\W|$)", ' ', get)
+                    get = (re.sub(r"[\t!#$@%^*+=`~/]+| +", ' ', get)).strip(' ') #Strip any anomalies
+                    get = re.sub(r"(?<=2)(ND)|(?<=[4-9]|0)(TH|RTH)", '', get) #Strip any char of ordinal numbers
+                    get = re.sub(r"(?<=[^1]3)(RD)", '', get); get = re.sub(r"(?<=11)(TH)", '', get)
+        else:
+            get = (re.sub(r"[\t!#$@%^*+=`~/]+| +", ' ', get)).strip(' ') #Strip any anomalies
+            get = re.sub(r"(?<=2)(ND)|(?<=[4-9]|0)(TH|RTH)", '', get) #Strip any char of ordinal numbers
+            get = re.sub(r"(?<=[^1]3)(RD)", '', get); get = re.sub(r"(?<=11)(TH)", '', get)
         if re.search(r"(\W|^)(ST|SNT)\W", get): #Check for 'Saint'
             get1 = re.sub(r"(\W|^)(ST|SNT)\W", ' ', get)
             saintFlag = True
-            new_street = self.searchCycle(get1, saintFlag)
+            new_street = self._search_cycle(get1, saintFlag)
             if new_street == "OTHER":
                 saintFlag = False
-                new_street = self.searchCycle(get, saintFlag)
+                new_street = self._search_cycle(get, saintFlag)
         else:
-            new_street = self.searchCycle(get, saintFlag)
+            new_street = self._search_cycle(get, saintFlag)
         new_street = re.sub(r"^FT\W| FT\W", "FORT ", new_street) #Replace 'FT' with 'FORT'
         new_street = re.sub(r"(?<=1)ST", '', new_street) #Strip 1st ordinal number
         if re.search(r"\d+", new_street): #Apply ordinal numbers
-            ordNum = self.ordinalAdd(str(re.search(r"\d+", new_street).group()))
+            ordNum = self._ordinal_add(str(re.search(r"\d+", new_street).group()))
             new_street = re.sub(r"\d+", ordNum, new_street)
 
         if log != '': #Write to new or specfied file
@@ -257,8 +300,6 @@ class GeoLiberator:
 
     def getAddress(self, log=''):
         get = (self.addr).upper(); new_addr = '' #Uppercase and create new address to return
-        get = (re.sub(r"[\t!#$@%^*+=`~/]| +", ' ', get)).strip(' ') #Strip any anomalies
-        get = re.sub(r"(?<=\d)(ND|RD|TH|RTH)", '', get) #Strip any char of ordinal numbers
         gS = GeoLiberator(get).getStreet()
         gAN = GeoLiberator(get).getAddressNum()
         if gAN != "OTHER" and gS != "OTHER":
@@ -285,6 +326,8 @@ def file_len(file_name):
 #Takes text file as input and switch argument to determine which address property to be standardized
 def autoGeoLiberate(file_path, parse="address", write=''):
     mode = True
+    if not re.search(r"address|number|street", parse):
+        raise ArgumentError(reason[0])
     if write != '':
         mode = False
     with open(file_path) as f:
@@ -293,16 +336,18 @@ def autoGeoLiberate(file_path, parse="address", write=''):
             if sys.argv[1] == '--status' or sys.argv[1] == "-S":
                 FL = file_len(file_path)
                 barIncr = int(FL * .025); barNum = 0; dashNum = 40; c = 0; lc = 0
+                print('|' + ('-' * 40) + '|' + " [0.00%]", end=''); sys.stdout.flush()
                 for line in lines:
                     perc = (lc/FL)
-                    bar = '|' + ('█' * barNum) + ('-' * dashNum) + '|' + " [{:>.2%}]".format(perc)
+                    bar = '\r|' + ('█' * barNum) + ('-' * dashNum) + '|' + " [{:>.2%}]".format(perc)
+                    print(bar, end=''); sys.stdout.flush()
                     c += 1; lc += 1
                     if c == barIncr:
-                       if barNum < 39:
-                           c = 0; barNum += 1; dashNum -= 1
-                           print(f"\r{bar}", end=''); sys.stdout.flush()
+                        if barNum < 39:
+                            c = 0; barNum += 1; dashNum -= 1
+                            print(bar, end=''); sys.stdout.flush()
                     elif lc == FL:
-                       print('\r|' + ('█' * 40) + '|' + " [100%]  "); sys.stdout.flush()
+                        print('\r|' + ('█' * 40) + '|' + " [100%]  "); sys.stdout.flush()
                     adr = GeoLiberator(str(line))
                     if parse.lower() == "address":
                         adr.getAddress(log=write)
@@ -310,6 +355,8 @@ def autoGeoLiberate(file_path, parse="address", write=''):
                         adr.getAddressNum(log=write)
                     elif parse.lower() == "street":
                         adr.getStreet(log=write)
+                    elif parse.lower() == "state":
+                        adr._get_state(log=write)
         else:
             if mode == False:
                 print("Running...")
@@ -321,6 +368,8 @@ def autoGeoLiberate(file_path, parse="address", write=''):
                     out = adr.getAddressNum(log=write)
                 elif parse.lower() == "street":
                     out = adr.getStreet(log=write)
+                elif parse.lower() == "state":
+                    out = adr._get_state(log=write)
                 if mode == True:
                     print(out)
             print("Done!")
@@ -328,16 +377,15 @@ def autoGeoLiberate(file_path, parse="address", write=''):
 #Takes address as input and switch argument to determine which address property to be standardized
 def geoLiberate(addr, parse="address"):
     adr = GeoLiberator(str(addr))
-    if parse.lower() == "address":
-        out = adr.getAddress()
-    elif parse.lower() == "number":
-        out = adr.getAddressNum()
-    elif parse.lower() == "street":
-        out = adr.getStreet()
-    print(out)
-
-##t1 = time.process_time_ns()
-##total = t1 - t0
-##if __name__ == "__main__":
-##    print(f"Timestamp 1: {t0} n/s\nTimestamp 2: {t1} n/s")
-##    print("Module Time Elapsed:", total, "nanoseconds")
+    try:
+        if parse.lower() == "address":
+            out = adr.getAddress()
+        elif parse.lower() == "number":
+            out = adr.getAddressNum()
+        elif parse.lower() == "street":
+            out = adr.getStreet()
+        elif parse.lower() == "state":
+            out = adr._get_state()
+        print(out)
+    except (AttributeError, UnboundLocalError):
+        raise ArgumentError(reason[0])
